﻿interface IReader {
	char ReadChar();
	void Close();
}

interface IWriter {
	void WriteChar(char c);
	void Close();
}

class ConsoleReaderWriter : IReader, IWriter {
	public bool Closed { get; private set; } = false;

	public char ReadChar() {
		if (Closed) throw new InvalidOperationException("Reading from a closed ConsoleReaderWriter.");
		return (char) Console.Read();
	}

	public void WriteChar(char c) {
		if (Closed) throw new InvalidOperationException("Writing to a closed ConsoleReaderWriter.");
		Console.Write(c);
	}

	public void Close() {
		Closed = true;
	}
}

class Program {
	static void Main(string[] args) {
		{
			Console.WriteLine("1) Creating ConsoleReaderWriter ...");
			var consoleReaderWriter = new ConsoleReaderWriter();
			Console.WriteLine($"  Closed? {consoleReaderWriter.Closed}");

			Console.WriteLine("2) Waiting for input (enter a char and then press ENTER) ...");
			var data = DoReadProcessing(consoleReaderWriter);
			Console.WriteLine($"  Closed? {consoleReaderWriter.Closed}");

			Console.WriteLine("3) Generating output ...");
			DoWriteProcessing(consoleReaderWriter, data);
			Console.WriteLine();
			Console.WriteLine($"  Closed? {consoleReaderWriter.Closed}");
		}
	}

	static char DoReadProcessing(IReader reader) {
		var result = reader.ReadChar();
		reader.Close();
		return result;
	}

	static void DoWriteProcessing(IWriter writer, char c) {
		writer.WriteChar('>');
		writer.WriteChar(c);
		writer.Close();
	}
}