﻿using System;

interface I1 {
	void m();
}

interface I2 {
	void m();
}

class A : I1 {
	public void m() {
		Console.WriteLine("A.m()");
	}
}

class B : I1, I2 {
	public void m() {
		Console.WriteLine("B.m()");
	}
}

class C : A, I2 {
	public new void m() {
		Console.WriteLine("C.m()");
	}
}

class D : I1, I2 {
	public void m() {
		Console.WriteLine("D.m()");
	}

	void I1.m() {
		Console.WriteLine("D.I1.m()");
	}
}

class X<T> where T : I1 {
	public static void m(T t) {
		t.m();
	}
}

class Y<T> where T : I1, I2 {
	public static void m(T t) {
		Console.WriteLine($"Y<{typeof(T).Name}>.m() called:");
		// Compile error: The call is ambiguous between the following methods or properties: 'I1.m()' and 'I2.m()'
		// t.m();

		((I1) t).m();
		((I2) t).m();
	}
}

class Program {
	static void Main(string[] args) {
		Console.WriteLine("Y<T> tests:");
		// Compile error: The type 'A' cannot be used as type parameter 'T' in the generic type or method 'Y<T>'.
		// Y<A>.m(new A());		

		Y<B>.m(new B()); Console.WriteLine();
		Y<C>.m(new C()); Console.WriteLine();
		Y<D>.m(new D()); Console.WriteLine();

		Console.WriteLine("Direct m call:");

		new D().m(); Console.WriteLine();

		I1 i1 = new D();
		i1.m();

		((I1) new D()).m(); Console.WriteLine();

		Console.WriteLine("X<T> tests:");

		X<A>.m(new A()); Console.WriteLine();
		X<C>.m(new C()); Console.WriteLine();
	}
}

