﻿using System.Diagnostics;

struct X<T> where T : class {
	private T t;

	public static implicit operator T(X<T> x) {
		return x.t;
	}

	public static implicit operator X<T>(T t) {
		X<T> x;
		x.t = t;
		return x;
	}
}

public class A {
	public int Value;
}

class ArrayProcessor {
	public A[] Data = new A[Program.Count];

	public void Initialize() {
		for (int i = 0; i < Data.Length; i++) {
			Data[i] = new A { Value = Program.Random.Next(Program.Count) };
		}
	}

	public void Process() {
		for (int i = 1; i < Data.Length; i++) {
			A item = Data[i];
			int j = i - 1;
			while ((j >= 0) && (Data[j].Value > item.Value)) {
				Data[j + 1] = Data[j];
				j--;
			}
			Data[j + 1] = item;
		}
	}
}

class ValueArrayProcessor {
	public X<A>[] Data = new X<A>[Program.Count];

	public void Initialize(ArrayProcessor ap) {
		for (int i = 0; i < Data.Length; i++) {
			Data[i] = new A { Value = ap.Data[i].Value };
		}
	}

	public void Process() {
		for (int i = 1; i < Data.Length; i++) {
			A item = Data[i];
			int j = i - 1;
			while ((j >= 0) && (((A) Data[j]).Value > item.Value)) {    // The only change: "((A) Data[j]).Value" instead of "Data[j].Value"
				Data[j + 1] = Data[j];
				j--;
			}
			Data[j + 1] = item;
		}
	}
}

class Program {
	public const int Count = 50000;
	public static Random? Random;

	static void Main(string[] args) {
		Console.WriteLine(Environment.Is64BitProcess);

		// Random.Next(1);

		Console.WriteLine("Warming up ...");
		Program.Random = new Random(42);
		var ap1 = new ArrayProcessor();
		ap1.Initialize();
		ap1.Process();
		var ap2 = new ValueArrayProcessor();
		ap2.Initialize(ap1);
		ap2.Process();
		ap1 = null;
		ap2 = null;
		GC.Collect(GC.MaxGeneration, GCCollectionMode.Forced, true);

		Console.WriteLine("Generating data ...");
		Program.Random = new Random(42);

		ap1 = new ArrayProcessor();
		ap1.Initialize();

		ap2 = new ValueArrayProcessor();
		ap2.Initialize(ap1);

		Console.WriteLine("Running AP:");
		var sw1 = Stopwatch.StartNew();
		ap1.Process();
		sw1.Stop();
		Console.WriteLine(sw1.Elapsed);

		Console.WriteLine("Running VAP with X<A>:");
		var sw2 = Stopwatch.StartNew();
		ap2.Process();
		sw2.Stop();
		Console.WriteLine(sw2.Elapsed);

		Console.WriteLine($"Speed up: {100 - (sw2.ElapsedMilliseconds / (double) sw1.ElapsedMilliseconds) * 100} %");
	}
}