﻿using System;
using System.Collections.Generic;

namespace CovariantReturnsInCsharp9 {

	record A {	// Could be class as well.
		public int Id { get; set; }
	}

	record B : A {  // Could be class as well (if A is a class and not a record).
		public string Name { get; set; }
	}

	class X {
		public virtual A GetA() => new A { Id = 42 };
	}

	class Y : X {
		public override A GetA() => new B { Id = base.GetA().Id + 100, Name = "Pavel" };
	}

	// C# 9.0:
	class Z : X {
		public override B GetA() => new B { Id = base.GetA().Id + 1000, Name = "Petr" };
	}

	class Program {
		static void Main(string[] args) {
			List<X> list = new() { new X(), new Y(), new Z() };

			for (int i = 0; i < list.Count; i++) {
				Console.WriteLine($"Type: {list[i].GetA().GetType()}, value: {list[i].GetA()}");
			}

			Y y = new Y();
			var value = y.GetA();
			Console.WriteLine(value.Id);
			var valueB = (B) value;

			Z z = new Z();
			var value2 = z.GetA();
			Console.WriteLine(value2.Name);
		}
	}

	/* 
	class X {
		public virtual A GetA() => new A { Id = DefaultId };
		protected int DefaultId => 42;
	}

	class Y : X {
		public override A GetA() => new B { Id = DefaultId, Name = "Pavel" };
	}

	*/
}
